"use strict";
/**
 * @license Apache-2.0
 * @copyright Cube Dev, Inc.
 * @fileoverview
 * prepareAnnotation function and related types definition.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.prepareAnnotation = exports.annotation = void 0;
const ramda_1 = __importDefault(require("ramda"));
const shared_1 = require("@cubejs-backend/shared");
const toConfigMap_1 = require("./toConfigMap");
const enums_1 = require("../types/enums");
/**
 * Returns annotations by MetaConfigMap and cube's member type.
 */
const annotation = (configMap, memberType) => (member) => {
    const [cubeName, fieldName] = member.expression ? [member.cubeName, member.name] : member.split('.');
    const memberWithoutGranularity = [cubeName, fieldName].join('.');
    const cubeConfig = configMap[cubeName];
    const config = cubeConfig && cubeConfig[memberType]
        .find(m => m.name === memberWithoutGranularity);
    if (!config) {
        return undefined;
    }
    return [typeof member === 'string' ? member : memberWithoutGranularity, {
            title: config.title,
            shortTitle: config.shortTitle,
            description: config.description,
            type: config.type,
            format: config.format,
            meta: config.meta,
            ...(memberType === enums_1.MemberType.MEASURES ? {
                drillMembers: config.drillMembers,
                drillMembersGrouped: config.drillMembersGrouped
            } : {}),
            ...(memberType === enums_1.MemberType.DIMENSIONS && config.granularities ? {
                granularities: config.granularities || [],
            } : {}),
        }];
};
exports.annotation = annotation;
/**
 * Returns annotations object by MetaConfigs and query.
 */
function prepareAnnotation(metaConfig, query) {
    const configMap = (0, toConfigMap_1.toConfigMap)(metaConfig);
    const dimensions = (query.dimensions || []);
    return {
        measures: ramda_1.default.fromPairs((query.measures || []).map(annotation(configMap, enums_1.MemberType.MEASURES)).filter(a => !!a)),
        dimensions: ramda_1.default.fromPairs(dimensions
            .map(annotation(configMap, enums_1.MemberType.DIMENSIONS))
            .filter(a => !!a)),
        segments: ramda_1.default.fromPairs((query.segments || [])
            .map(annotation(configMap, enums_1.MemberType.SEGMENTS))
            .filter(a => !!a)),
        timeDimensions: ramda_1.default.fromPairs(ramda_1.default.unnest((query.timeDimensions || [])
            .filter(td => !!td.granularity)
            .map(td => {
            const an = annotation(configMap, enums_1.MemberType.DIMENSIONS)(`${td.dimension}.${td.granularity}`);
            let dimAnnotation;
            if (an) {
                let granularityMeta;
                if ((0, shared_1.isPredefinedGranularity)(td.granularity)) {
                    granularityMeta = {
                        name: td.granularity,
                        title: td.granularity,
                        interval: `1 ${td.granularity}`,
                    };
                }
                else if (an[1].granularities) {
                    // No need to send all the granularities defined, only those make sense for this query
                    granularityMeta = an[1].granularities.find(g => g.name === td.granularity);
                }
                const { granularities: _, ...rest } = an[1];
                dimAnnotation = [an[0], { ...rest, granularity: granularityMeta }];
            }
            // TODO: deprecated: backward compatibility for
            // referencing time dimensions without granularity
            if (dimensions.indexOf(td.dimension) !== -1) {
                return [dimAnnotation].filter(a => !!a);
            }
            const dimWithoutGranularity = annotation(configMap, enums_1.MemberType.DIMENSIONS)(td.dimension);
            if (dimWithoutGranularity && dimWithoutGranularity[1].granularities) {
                // no need to populate granularities here
                dimWithoutGranularity[1].granularities = undefined;
            }
            return [dimAnnotation].concat([dimWithoutGranularity])
                .filter(a => !!a);
        }))),
    };
}
exports.prepareAnnotation = prepareAnnotation;
exports.default = prepareAnnotation;
//# sourceMappingURL=prepareAnnotation.js.map